@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 ##############################################################
 # smscon_service - remote control daemon for Nokia N900 phone#
-# $Id: smscon_service.py 150 2012-01-15 19:54:20Z yablacky $
+# $Id: smscon_service.py 156 2012-01-20 20:12:07Z yablacky $
 ##############################################################
 
 # standard modules
@@ -39,14 +39,19 @@ LOGGER = smscon.LOGGER = smscon.StartLogging('DAEMON', withConsole = False)
 
 PingCountMax                = 3                                 # max number of pings for connection status check
 LastKnownGoodConfigFile     = '.%s_lkg' % smscon.NAME
+GpsModeLOCATION             = 'location'
+GpsModeTRACKING             = 'tracking'
 
 #############################################
 #    variables
 
-GpsCoordinateList           = []
+GpsCoordinateList           = None
 GpsMode                     = False                             # False if not active or mode-name if GPS device is enabled
 GpsIsTest                   = False                             # true if GPS device is enabled from test mode command
 GpsStartTime                = 0
+GpsTimeoutTime              = 0
+GpsLocationBasisTime        = 0
+GspTrackingBasisTime        = 0
 GpsControl                  = None
 
 ConnectionType              = None 
@@ -305,6 +310,7 @@ def SendToGSMmodem(RecipientNumber, Message, SuccessLogMsg, TryCount = None):
 def DeOctifyMessage(inputBytes):
     """
     Decode inputBytes from SMS message according to GSM 03.38, GSM 7 bit default alphabet.
+    @param list inputBytes Bytes containing packed 7 bit chars (from "PDUMessage").
     @return string.
     """
     outputCodes = []
@@ -583,7 +589,7 @@ def ProcessCommand(Command, SenderPhoneNumber=None, CommandOrigin=None, ExtraInf
             SetNetworkConnectionType('GPRS') # enable GPRS data network
             behaviourChanged = SetAutoConnectBehaviour('GPRS') # disable switching to other network
 
-        EnableGPStracking('single')
+        EnableGpsAcquireMode(GpsModeLOCATION)
 
         if behaviourChanged:
             SetAutoConnectBehaviour(ConnectionBehaviour)
@@ -623,10 +629,10 @@ def ProcessCommand(Command, SenderPhoneNumber=None, CommandOrigin=None, ExtraInf
         if not IsInternetReachable():
             SetNetworkConnectionType('GPRS') # enable GPRS data network
             SetAutoConnectBehaviour('GPRS') # disable switching to other network
-        EnableGPStracking('multi')
+        EnableGpsAcquireMode(GpsModeTRACKING)
 
     elif Command == smscnf.COM_TRACKOFF:
-        DisableGPStracking()
+        DisableGpsAcquiring()
         SetAutoConnectBehaviour(ConnectionBehaviour)
 
     elif Command == smscnf.COM_CUSTOM:
@@ -1095,7 +1101,7 @@ def GetGPRSConnectionStatus():
 
 def SetNetworkConnectionType(NewType):
     """
-    Connect to GPRS data network.
+    Connect to GPRS or WLAN data network.
     @param string Network type: GPRS|WLAN
     @return bool Success indicator
     """
@@ -1432,49 +1438,43 @@ def DisableSSHcontrol(Verbose = True):
 #   GPS
 #############################################
 
-def EnableGPStracking(Mode):
+def EnableGpsAcquireMode(Mode):
     """
     Start the GPS device mainloop.
     """
-    global GpsMode
-    if not GpsMode:
-        GpsMode = Mode
-        # enable GPS device control
-        gobject.idle_add(GPSstart, GpsControl)
-    else:
-        LOGGER.warning('GPS device already active in %s mode.' % GpsMode)
+    LOGGER.debug('Enabling GPS device for %s mode (current mode is %s).' % (Mode, GpsMode))
+    gobject.idle_add(GPSstart, Mode)
 
-def DisableGPStracking():
+def DisableGpsAcquiring():
     """
     Stop the GPS device mainloop.
     """
-    GpsData = GpsControl
+    global GpsMode, GpsControl
 
-    global GpsMode
     if GpsMode:
+        LOGGER.info('Stopped GPS device from %s mode.' % GpsMode)
         GpsMode = False
         # stop the GPS device
-        GpsData.stop()
-        LOGGER.info('Stopped acquiering GPS coordinates.')
+        GpsControl.stop()
     else:
         LOGGER.info('GPS device already stopped.')
 
-def GPSstart(GpsData):
+def GPSstart(Mode):
     """
-    Start the GPS device.
+    Start the GPS device in given Mode or switch already started device to given Mode.
     """
-    if GpsMode == 'single':
-        LOGGER.info('Starting GPS device in location mode.')
-    elif GpsMode == 'multi':
-        LOGGER.info('Starting GPS device in tracking mode.')
-    else:
-        LOGGER.warning('Starting GPS device in %s mode.' % str(GpsMode))
+    global GpsMode, GpsControl, GpsStartTime, GpsTimeoutTime, GpsLocationBasisTime, GspTrackingBasisTime
 
-    global GpsStartTime
-    GpsStartTime = time.time()
-    GpsData.start()
-    
-    return False
+    if GpsMode:
+        LOGGER.info('Switching GPS device mainloop to %s mode (was %s).' % (Mode, GpsMode))
+        GpsMode = Mode
+    else:
+        LOGGER.info('Starting GPS device mainloop in %s mode.' % Mode)
+        GpsMode = Mode
+        GpsStartTime = GpsTimeoutTime = time.time()
+        GpsLocationBasisTime = GspTrackingBasisTime = 0
+        GpsControl.start()
+    return False    # single shot.
 
 def OnGPSstop(GpsControl, GpsData):
     """
@@ -1495,84 +1495,123 @@ def OnGPSchanged(GpsDevice, GpsData):
     """
     Get GPS device data.
     """
-    global GpsCoordinateList
-    global GpsStartTime
+    global GpsCoordinateList, GpsStartTime, GpsTimeoutTime, GpsLocationBasisTime, GspTrackingBasisTime
 
     LOGGER.debug('OnGPSchanged() called for mode "%s".' % str(GpsMode))
-    if not GpsMode:
+    # The next check is essential because we may even get callbacks if another app turns GPS device on!
+    if not (GpsMode in (GpsModeLOCATION, GpsModeTRACKING)):
         return
 
-    TimePassedSeconds = time.time() - GpsStartTime
-    LOGGER.debug('GPS GpsStartTime = %s.' % GpsStartTime)
+    timeNow = time.time()
+
+    StartedSinceSeconds = timeNow - GpsStartTime
+    TimeoutSeconds      = timeNow - GpsTimeoutTime
+    LOGGER.debug('GpsStartTime = %s, StartedSinceSeconds = %.1f, TimeoutSeconds = %.1f'
+                                % (GpsStartTime, StartedSinceSeconds, TimeoutSeconds))
     LOGGER.debug('GpsDevice.status = %s' % GpsDevice.status) # LOCATION_GPS_DEVICE_STATUS_NO_FIX  or  LOCATION_GPS_DEVICE_STATUS_FIX
 
+    # Check if data is OK and log why it it not OK.
+
     GpsDataOK = True
     if GpsDataOK:
         GpsDataOK = (GpsDevice.fix[1] & location.GPS_DEVICE_LATLONG_SET) != 0
         if not GpsDataOK:
-            LOGGER.info('Waiting for GPS satellite fix (search time = %.1f sec.).' % TimePassedSeconds)
+            LOGGER.info('Waiting for GPS satellite fix (since %.1f of %d sec).' % (TimeoutSeconds, int(smscnf.GPSTIMEOUT)))
 
     if GpsDataOK:
         # GPS device has fix.
         Latitude  = GpsDevice.fix[4]
         Longitude = GpsDevice.fix[5]
-        Accuracy  = GpsDevice.fix[6]
+        Accuracy  = GpsDevice.fix[6]    # centimeters
 
-        # If just the accuracy is unknown then set it to a very high value (better than no coords et all)   
-        if GpsMode == 'multi' and str(Accuracy) == 'nan': 
-            Accuracy = 999999 * 100  # 999999 meters in centimeters. 
+        if str(Accuracy) != 'nan':
+            Accuracy  = Accuracy / 100  # centimeters to meters
+        elif GpsMode == GpsModeTRACKING: 
+            Accuracy = 999999  # 999999 meters. A very high value should be better than no coords et all.
 
         # Check if coordinates acquired by GPS device are valid
         GpsDataOK = str(Latitude) != 'nan' and str(Longitude) != 'nan' and str(Accuracy) != 'nan'
         if not GpsDataOK:
-            LOGGER.info('Waiting for valid GPS coordinate (search time = %.1f sec.).' % TimePassedSeconds)
+            LOGGER.info('Waiting for valid GPS coordinate (since %.1f of %d sec).' % (TimeoutSeconds, int(smscnf.GPSTIMEOUT)))
 
-        Accuracy  = Accuracy / 100  # centimeters to meters
+    if GpsDataOK:
+        GpsTimeoutTime = timeNow  # restart timeout
+    IsTimeoutDetected = TimeoutSeconds > int(smscnf.GPSTIMEOUT)
 
-    if not GpsDataOK:
-        # check GPS device timout
-        if TimePassedSeconds >= int(smscnf.GPSTIMEOUT):
-            LOGGER.error(     'GPS device timeout after %.1f sec.' % TimePassedSeconds)
-            RevealThat('GPS', 'GPS device timeout after %.1f sec.' % TimePassedSeconds, smscnf.GPSSEND)                    
+    # Now handle things differently depending on GpsMode:
+
+    if GpsMode == GpsModeLOCATION:
+        LocationTimeoutSeconds = max(int(smscnf.GPSINTERVAL), 30) * int(smscnf.GPSPOLLING)
+        LocationSinceSeconds = timeNow - GpsLocationBasisTime 
+        if GpsDataOK:
+
+            if type(GpsCoordinateList).__name__ != 'list':
+                GpsCoordinateList = []
+                GpsLocationBasisTime = timeNow
+                LOGGER.info('Start collecting min. %s GPS coordinate(s) for max. %d sec...' % (smscnf.GPSPOLLING, LocationTimeoutSeconds))
+                # Ignore the 1st received coordinate because this might be just the last coordinate
+                # from last long-time-ago fix and has nothing to do with current position.  
+                LOGGER.info('GPS coordinate #%d (%f, %f, %.1f m.) after %.1f sec since start. Ignored.'
+                                % (len(GpsCoordinateList), Latitude, Longitude, Accuracy, StartedSinceSeconds) )
+                return  # gather more coordinates
+
+            GpsCoordinateList.append( (Latitude, Longitude, Accuracy) )
+            LOGGER.info('GPS coordinate #%d (%f, %f, %.1f m.) after %.1f sec since fix.'
+                                % (len(GpsCoordinateList), Latitude, Longitude, Accuracy, LocationSinceSeconds) )                
+
+            if len(GpsCoordinateList) < int(smscnf.GPSPOLLING):
+                LOGGER.debug('gather more coordinates (GPSPOLLING count).')
+                return  # gather more coordinates
+            ReportTheLocation = True
+        else:
+            # If this is timeout but we have some coords already, report them.
+            ReportTheLocation = IsTimeoutDetected and len(GpsCoordinateList or []) > 0
+
+        if ReportTheLocation:
+            # Sort GPS coordinate list on 'Accuracy'
+            GpsCoordinateList = sorted( GpsCoordinateList, key=itemgetter(2) )
+            Latitude, Longitude, Accuracy = GpsCoordinateList[0]
+            GpsDataOK = True
+
+            # If phone is in move, then GSP change reports may arrive faster then as
+            # specified in smscnf.GPSINTERVAL. If the GPS device was started in this
+            # situation, we get a lot of less than possibe accurate coords very fast.
+            # If best accuracy is not in tolecance and we still have time, try further:
+            if Accuracy > int(smscnf.get('GPSMAXDEVIATION', 50)) and LocationSinceSeconds < LocationTimeoutSeconds:
+                LOGGER.debug('gather more coordinates (Accuracy > GPSMAXTOLERANCE and %d < %d sec).'
+                                                                % (LocationSinceSeconds, LocationTimeoutSeconds))
+                return  # gather more coordinates
+
+            LOGGER.info( 'Most accurate GPS coordinate out of %d = (%f, %f, %.1f m.) after %.1f sec since start.'
+                            % (len(GpsCoordinateList), Latitude, Longitude, Accuracy, StartedSinceSeconds) )
+            GpsCoordinateList = None
             # stop the GPS device
             GpsData.stop()
-        LOGGER.debug('GpsData not OK after %.1f sec.' % TimePassedSeconds)
+
+    elif GpsMode == GpsModeTRACKING:
+        if GpsDataOK:
+            if GspTrackingBasisTime and timeNow < GspTrackingBasisTime + int(smscnf.GPSINTERVAL):
+                LOGGER.debug('gather more coordinates (data not OK but ealier then GPSINTERVAL).')
+                return
+            GspTrackingBasisTime = timeNow
+            LOGGER.info( 'GPS coordinate acquired (%f, %f, %.1f m.) after %.1f sec since start.'
+                            % (Latitude, Longitude, Accuracy, StartedSinceSeconds) )
+
+    # Common report part. 
+
+    if not GpsDataOK:
+        if IsTimeoutDetected:
+            LOGGER.error(     'GPS device timeout after %.1f sec.' % StartedSinceSeconds)
+            RevealThat('GPS', 'GPS device timeout after %.1f sec.' % StartedSinceSeconds, smscnf.GPSSEND)                    
+            # stop the GPS device
+            GpsData.stop()
+        LOGGER.debug('gather more coordinates (data not OK but no timeout).')
         return
 
-    # Now we have really data, adjust timeout timer
-    GpsStartTime = time.time()
-
-    if GpsMode == 'single':
-        if len(GpsCoordinateList) == 0:
-            LOGGER.info('Start collecting %s GPS coordinate(s)...' % smscnf.GPSPOLLING)
-
-        GpsCoordinateList.append( (Latitude, Longitude, Accuracy) )
-        LOGGER.info( 'GPS coordinate #%d acquired (%f, %f, %.1f m.).'
-                        % (len(GpsCoordinateList), Latitude, Longitude, Accuracy) )                
-        # We will dismiss the 1st received coordinate because this might be just the
-        # last coordinate from last long-time-ago fix and has nothing to do with
-        # current position. So effectively accuire one more coordinate than specified:  
-        if len(GpsCoordinateList) < int(smscnf.GPSPOLLING) + 1:
-            return
-        GpsCoordinateList[0:1] = []     # Dismiss the 1st received coordinate. See comment above.
-        # sort GPS coordinate list on 'Accuracy'
-        GpsCoordinateList = sorted( GpsCoordinateList, key=itemgetter(2) )
-        Latitude, Longitude, Accuracy = GpsCoordinateList[0]
-        LOGGER.info( 'Most accurate GPS coordinate out of %d = (%f, %f, %.1f m.).'
-                        % (len(GpsCoordinateList), Latitude, Longitude, Accuracy) )
-
-        GpsCoordinateList = []
-        # stop the GPS device
-        GpsData.stop()
-
-    # tracking mode
-    elif GpsMode == 'multi':
-        LOGGER.info( 'GPS coordinate acquired (%f, %f, %.1f m.).' % (Latitude, Longitude, Accuracy) )
-
     GpsText = ['Lat: %f\n' % Latitude,
                'Long: %f\n' % Longitude,
                'Accuracy: %.1f mtr\n' % Accuracy,
-               'Duration: %.1f sec\n' % TimePassedSeconds]
+               'Duration: %.1f sec\n' % StartedSinceSeconds]
 
     LinkText = ['<a href="http://maps.google.com/maps?q=%f,%f">GoogleMap</a>\n'% (Latitude, Longitude)]
 
@@ -1753,11 +1792,11 @@ def DoCommandLine(args):
 
                 global GpsIsTest
                 GpsIsTest = True
-                
-                if Func == 'gps1': # test mode for gps (location mode)
-                    EnableGPStracking('single')
-                elif Func == 'gps2': # test mode for gps (tracking mode)
-                    EnableGPStracking('multi')
+
+                if Func == 'gps1':
+                    EnableGpsAcquireMode(GpsModeLOCATION)
+                elif Func == 'gps2':
+                    EnableGpsAcquireMode(GpsModeTRACKING)
                 return CommandLineStatus
 
             if Func == 'ssh': # test mode for ssh connection
@@ -2083,18 +2122,22 @@ def ApplyConfiguration():
     """
     global smscnf, GpsControl
     
-    # Set GPS method & most suitable interval
+    # Set GPS method & most suitable interval. Do not select intervals above 30 seconds:
+    # In tracking mode we check the GPSINTERVAL ourself and in location mode the longer
+    # intervals do not make very much sense anyway. Keep in mind that the actual interval
+    # is also influenced by other applications that are using the GPS device! 
 
     gps_interval = int(smscnf.GPSINTERVAL)
-    if   gps_interval <= 10:  gps_interval = 10;  pref_interval = location.INTERVAL_10S
-    elif gps_interval <= 20:  gps_interval = 20;  pref_interval = location.INTERVAL_20S
-    elif gps_interval <= 30:  gps_interval = 30;  pref_interval = location.INTERVAL_30S
-    elif gps_interval <= 60:  gps_interval = 60;  pref_interval = location.INTERVAL_60S
-    else:                     gps_interval = 120; pref_interval = location.INTERVAL_120S
+    if   gps_interval <= 1:   pref_interval = location.INTERVAL_1S
+    elif gps_interval <= 2:   pref_interval = location.INTERVAL_2S
+    elif gps_interval <= 5:   pref_interval = location.INTERVAL_5S
+    elif gps_interval <= 10:  pref_interval = location.INTERVAL_10S
+    elif gps_interval <= 20:  pref_interval = location.INTERVAL_20S
+    else:                     pref_interval = location.INTERVAL_30S
     GpsControl.set_properties(#preferred_method   = location.METHOD_GNSS|location.METHOD_AGNSS,
                               preferred_method   = location.METHOD_USER_SELECTED,
                               preferred_interval = pref_interval)
-    LOGGER.info('Preferred GPS query interval is %d sec.' % gps_interval)
+    LOGGER.info('GPS query interval is %d sec.' % gps_interval)
 
 def LoadConfiguration():
     """"
