@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 ##############################################################
 # smscon - remote control library for Nokia N900 phone       #
-# $Id: smsconlib.py 150 2012-01-15 19:54:20Z yablacky $
+# $Id: smsconlib.py 155 2012-01-20 20:11:27Z yablacky $
 ##############################################################
 
 # standard modules
@@ -34,7 +34,7 @@ NAME = 'smscon'
 VERSION = '999.1'   # for trunk and private builds etc.
 
 NAME, VERSION = getSysBuildNameAndVersion(NAME, VERSION,
-    headerInfo  = '$Header: https://vcs.maemo.org/svn/smscon/branches/smscon_0.9.14/src/opt/smscon/smsconlib.py 150 2012-01-15 19:54:20Z yablacky $',
+    headerInfo  = '$Header: https://vcs.maemo.org/svn/smscon/branches/smscon_0.9.15/src/opt/smscon/smsconlib.py 155 2012-01-20 20:11:27Z yablacky $',
     dirLevels   = 3 )
     
 XDEBUG   = False        # cross debug (use only if not running on phone)
@@ -814,8 +814,9 @@ class UserConfiguration(Configuration):
 
         # GPS settings
         'GPSTIMEOUT':       (_IntPattern,       '1800',             _OPT_PLAIN, '# Seconds until give up acquiring GPS coordinates:'),
-        'GPSPOLLING':       (_IntPattern,       '3',                _OPT_PLAIN, '# Number of GPS coordinate acquisitions for COM_LOCATIONs best of:'),
-        'GPSINTERVAL':      (_IntPattern,       '60',               _OPT_PLAIN, '# Seconds between GPS coordinate acquisitions for COM_TRACKON and COM_LOCATION (10..120:10|20|30|60|120)'),
+        'GPSPOLLING':       (_IntPattern,       '8',                _OPT_PLAIN, '# Number of minimum GPS coordinate acquisitions for COM_LOCATIONs best of:'),
+        'GPSINTERVAL':      (_IntPattern,       '10',               _OPT_PLAIN, '# Seconds between GPS coordinate acquisitions for COM_TRACKON and COM_LOCATION:'),
+        'GPSMAXDEVIATION':  (_IntPattern,       '50',               _OPT_PLAIN, '# Meters of minimim accuracy of a GPS coordinate for COM_LOCATION request:'),  
         'GPSSEND':          ('^(sms|email|both)$',          'sms',  _OPT_PLAIN, '# How to send GPS notifications (sms|email|both):'),
 
         # Miscellaneous settings
